#include <sstream>

#include "bms_define.h"
#include "bms_exception.h"


// -- BmsException -------------------------------------------------------
BmsException::BmsException( void ) :
std::domain_error( "BmsException" )
{
}

BmsException::~BmsException()
{
}


// -- BmsInternalException -----------------------------------------------
BmsInternalException::BmsInternalException( const char* file, unsigned int line ) :
file_( file ),
line_( line )
{
}

const char*
BmsInternalException::GetFile( void ) const
{
  return file_;
}

unsigned int
BmsInternalException::GetLine( void ) const
{
  return line_;
}


std::string
BmsInternalException::Message( void ) const
{
  std::ostringstream os;
  os << "G[܂B" << file_ << "(" << line_ << ")";
  return os.str();
}

// -- BmsOutOfRangeAccessException ---------------------------------------
BmsOutOfRangeAccessException::BmsOutOfRangeAccessException( const std::type_info& klass ) :
BmsWithCauseClassException( klass )
{
}

std::string
BmsOutOfRangeAccessException::Message( void ) const
{
  std::ostringstream os;
  os << this->GetCauseClass().name() << " ɂĔ͈͂𒴂ANZX܂B";
  return os.str();
}

// -- BmsInvalidWordValueUsedException -----------------------------------
BmsInvalidWordValueUsedException::BmsInvalidWordValueUsedException( int value ) :
value_( value )
{
}

int
BmsInvalidWordValueUsedException::GetValue( void ) const
{
  return value_;
}

std::string
BmsInvalidWordValueUsedException::Message( void ) const
{
  std::ostringstream os;
  os << "IuWFNg 00 ` ZZ ͈̔͊O̒lɂĂ當ɕϊ悤Ƃ܂ : " << value_;
  return os.str();
}

// -- BmsInvalidCharUsedAsWordException ----------------------------------
BmsInvalidCharUsedAsWordException::BmsInvalidCharUsedAsWordException( char msb, char lsb ) :
msb_( msb ),
lsb_( lsb )
{
}

char
BmsInvalidCharUsedAsWordException::GetMsb( void ) const
{
  return msb_;
}

char
BmsInvalidCharUsedAsWordException::GetLsb( void ) const
{
  return lsb_;
}

std::string
BmsInvalidCharUsedAsWordException::GetAsString( void ) const
{
  std::string tmp;
  tmp.append( msb_, 1 );
  tmp.append( lsb_, 1 );
  return tmp;
}

std::string
BmsInvalidCharUsedAsWordException::Message( void ) const
{
  std::ostringstream os;
  os << "0`9, A`Z ȊO̕IuWFNgɂ悤Ƃ܂B : " << this->GetAsString();
  return os.str();
}

// -- BmsInvalidStringConvertedAsBufferException -------------------------
BmsInvalidStringConvertedAsBufferException::BmsInvalidStringConvertedAsBufferException( const std::string& str ) :
str_( str )
{
}

const std::string&
BmsInvalidStringConvertedAsBufferException::GetString( void ) const
{
  return str_;
}

std::string
BmsInvalidStringConvertedAsBufferException::Message( void ) const
{
  std::ostringstream os;
  os << "IuWFNgzƂĕsȕIuWFNgzɕϊ悤Ƃ܂B : " << str_;
  return os.str();
}

// -- BmsTooLongStringConvetedAsBufferException --------------------------
BmsTooLongStringConvetedAsBufferException::BmsTooLongStringConvetedAsBufferException( const std::string& str ) :
str_( str )
{
}

const std::string&
BmsTooLongStringConvetedAsBufferException::GetString( void ) const
{
  return str_;
}

std::string
BmsTooLongStringConvetedAsBufferException::Message( void ) const
{
  std::ostringstream os;
  os << "\𒴂IuWFNgIuWFNgzɕϊ悤Ƃ܂B : " << str_;
  return os.str();
}


// -- BmsFileOpenException -----------------------------------------------
BmsFileOpenException::BmsFileOpenException( const std::string& filename, unsigned int error_number ) :
filename_( filename ),
error_number_( error_number )
{
}

const std::string&
BmsFileOpenException::GetFileName( void ) const
{
  return filename_;
}

unsigned int
BmsFileOpenException::GetErrorNumber( void ) const
{
  return error_number_;
}

std::string
BmsFileOpenException::Message( void ) const
{
  std::ostringstream os;
  os << "t@CĴɎs܂B" << filename_ << "(" << error_number_ << ")";
  return os.str();
}

// -- BmsFileAccessException -----------------------------------------------
BmsFileAccessException::BmsFileAccessException( const std::string& filename, unsigned int error_number ) :
filename_( filename ),
error_number_( error_number )
{
}

const std::string&
BmsFileAccessException::GetFileName( void ) const
{
  return filename_;
}

unsigned int
BmsFileAccessException::GetErrorNumber( void ) const
{
  return error_number_;
}

std::string
BmsFileAccessException::Message( void ) const
{
  std::ostringstream os;
  os << "t@C̃ANZXɎs܂B" << filename_ << "(" << error_number_ << ")";
  return os.str();
}


// -- BmsDuplicateHeaderException ----------------------------------------
BmsDuplicateHeaderException::BmsDuplicateHeaderException( const std::string& header ) :
header_( header )
{
}

const std::string&
BmsDuplicateHeaderException::GetHeader( void ) const
{
  return header_;
}

std::string
BmsDuplicateHeaderException::Message( void ) const
{
  std::ostringstream os;
  os << "wb_w肳܂Bwb_ : " << header_;
  return os.str();
}


// -- BmsDuplicateBarChangeException ----------------------------------------
BmsDuplicateBarChangeException::BmsDuplicateBarChangeException( unsigned int bar ) :
bar_( bar )
{
}

unsigned int
BmsDuplicateBarChangeException::GetBar( void ) const
{
  return bar_;
}

std::string
BmsDuplicateBarChangeException::Message( void ) const
{
  std::ostringstream os;
  os << "ߒύXw肳܂Bߔԍ : " << bar_;
  return os.str();
}


// -- BmsParseException --------------------------------------------------
BmsParseException::BmsParseException( unsigned int line ) :
line_( line )
{
}

unsigned int
BmsParseException::GetLine( void ) const
{
  return line_;
}


// -- BmsParseInvalidBarChangeValueException -----------------------------
BmsParseInvalidBarChangeValueException::BmsParseInvalidBarChangeValueException( unsigned int line, const std::string& str ) :
BmsParseException( line ),
str_( str )
{
}

const std::string&
BmsParseInvalidBarChangeValueException::GetString( void ) const
{
  return str_;
}

std::string
BmsParseInvalidBarChangeValueException::Message( void ) const
{
  std::ostringstream os;
  os << "ߒύX̒lsłBsȕ : " << str_;
  return os.str();
}


// -- BmsParseInvalidEndAsObjectArrayException ---------------------------
BmsParseInvalidEndAsObjectArrayException::BmsParseInvalidEndAsObjectArrayException( unsigned int line ) :
BmsParseException( line )
{
}

std::string
BmsParseInvalidEndAsObjectArrayException::Message( void ) const
{
  std::ostringstream os;
  os << "IuWFNgz̏IsłB";
  return os.str();
}

// -- class BmsParseNoObjectArrayException -------------------------------
BmsParseNoObjectArrayException::BmsParseNoObjectArrayException( unsigned int line ) :
BmsParseException( line )
{
}

std::string
BmsParseNoObjectArrayException::Message( void ) const
{
  std::ostringstream os;
  os << "IuWFNgz񂪂܂B";
  return os.str();
}

// -- class BmsParseTooManyObjectException -------------------------------
BmsParseTooManyObjectException::BmsParseTooManyObjectException( unsigned int line ) :
BmsParseException( line )
{
}

std::string
BmsParseTooManyObjectException::Message( void ) const
{
  std::ostringstream os;
  os << "\𒴂IuWFNgIuWFNgzɕϊ悤Ƃ܂B";
  return os.str();
}

// -- BmsParseDuplicateHeaderException -----------------------------------
BmsParseDuplicateHeaderException::BmsParseDuplicateHeaderException( unsigned int line, const std::string& header ) :
BmsParseException( line ),
header_( header )
{
}

const std::string&
BmsParseDuplicateHeaderException::GetHeader( void ) const
{
  return header_;
}

std::string
BmsParseDuplicateHeaderException::Message( void ) const
{
  std::ostringstream os;
  os << "wb_w肳܂Bwb_ : " << header_;
  return os.str();
}

// -- BmsParseDuplicateBarChangeException --------------------------------
BmsParseDuplicateBarChangeException::BmsParseDuplicateBarChangeException( unsigned int line, unsigned int bar ) :
BmsParseException( line ),
bar_( bar )
{
}

unsigned int
BmsParseDuplicateBarChangeException::GetBar( void ) const
{
  return bar_;
}

std::string
BmsParseDuplicateBarChangeException::Message( void ) const
{
  std::ostringstream os;
  os << "ߒύXw肳܂Bߔԍ : " << bar_;
  return os.str();
}

// -- BmsParseInvalidCharException ---------------------------------------
BmsParseInvalidCharException::BmsParseInvalidCharException( unsigned int line, const std::string& str ) :
BmsParseException( line ),
str_( str )
{
}

const std::string&
BmsParseInvalidCharException::GetString( void ) const
{
  return str_;
}

// -- BmsParseInvalidCharAsHeaderOrBarException --------------------------
BmsParseInvalidCharAsHeaderOrBarException::BmsParseInvalidCharAsHeaderOrBarException( unsigned int line, const std::string& str ) :
BmsParseInvalidCharException( line, str )
{
}

std::string
BmsParseInvalidCharAsHeaderOrBarException::Message( void ) const
{
  std::ostringstream os;
  os << "wb_܂͏ߔԍƂĕsȕw肳܂B : " << this->GetString();
  return os.str();
}

// -- BmsParseInvalidCharAsHeaderKeyException ----------------------------
BmsParseInvalidCharAsHeaderKeyException::BmsParseInvalidCharAsHeaderKeyException( unsigned int line, const std::string& str ) :
BmsParseInvalidCharException( line, str )
{
}

std::string
BmsParseInvalidCharAsHeaderKeyException::Message( void ) const
{
  std::ostringstream os;
  os << "wb_Ƃĕsȕw肳܂B : " << this->GetString();
  return os.str();
}

// -- BmsParseInvalidCharAsBarException ----------------------------------
BmsParseInvalidCharAsBarException::BmsParseInvalidCharAsBarException( unsigned int line, const std::string& str ) :
BmsParseInvalidCharException( line, str )
{
}

std::string
BmsParseInvalidCharAsBarException::Message( void ) const
{
  std::ostringstream os;
  os << "ߔԍƂĕsȕw肳܂B : " << this->GetString();
  return os.str();
}

// -- BmsParseInvalidCharAsChannelException ------------------------------
BmsParseInvalidCharAsChannelException::BmsParseInvalidCharAsChannelException( unsigned int line, const std::string& str ) :
BmsParseInvalidCharException( line, str )
{
}

std::string
BmsParseInvalidCharAsChannelException::Message( void ) const
{
  std::ostringstream os;
  os << "`lԍƂĕsȕw肳܂B : " << this->GetString();
  return os.str();
}

// -- BmsParseInvalidCharAsColonException --------------------------------
BmsParseInvalidCharAsColonException::BmsParseInvalidCharAsColonException( unsigned int line, const std::string& str ) :
BmsParseInvalidCharException( line, str )
{
}

std::string
BmsParseInvalidCharAsColonException::Message( void ) const
{
  std::ostringstream os;
  os << "`lԍƃIuWFNgz؂RƂĕsȕw肳܂B : " << this->GetString();
  return os.str();
}

// -- BmsParseInvalidCharAsObjectArraryException -------------------------
BmsParseInvalidCharAsObjectArraryException::BmsParseInvalidCharAsObjectArraryException( unsigned int line, const std::string& str ) :
BmsParseInvalidCharException( line, str )
{
}

std::string
BmsParseInvalidCharAsObjectArraryException::Message( void ) const
{
  std::ostringstream os;
  os << "IuWFNgzƂĕsȕw肳܂B : " << this->GetString();
  return os.str();
}

// -- BmsParseInvalidRandomValueException --------------------------------
BmsParseInvalidRandomValueException::BmsParseInvalidRandomValueException( unsigned int line,
                                                                          const std::string& key,
                                                                          const std::string& value ) :
BmsParseException( line ),
key_( key ),
value_( value )
{
}

const std::string&
BmsParseInvalidRandomValueException::GetKey( void ) const
{
  return key_;
}

const std::string&
BmsParseInvalidRandomValueException::GetValue( void ) const
{
  return value_;
}

std::string
BmsParseInvalidRandomValueException::Message( void ) const
{
  std::ostringstream os;
  os << "_\ #" << key_ << " Ŏw肳ꂽlsłBl : " << value_;
  return os.str();
}

// -- BmsParseUnexceptedEndIfException -----------------------------------
BmsParseUnexceptedEndIfException::BmsParseUnexceptedEndIfException( unsigned int line ) :
BmsParseException( line )
{
}

std::string
BmsParseUnexceptedEndIfException::Message( void ) const
{
  std::ostringstream os;
  os << "#IF w肳ꂸ #ENDIF w肳܂B";
  return os.str();
}

// -- BmsParseNotFoundEndIfException -------------------------------------
BmsParseNotFoundEndIfException::BmsParseNotFoundEndIfException( unsigned int line ) :
BmsParseException( line )
{
}

std::string
BmsParseNotFoundEndIfException::Message( void ) const
{
  std::ostringstream os;
  os << "#ENDIF w肳ꂸɃt@CI܂B";
  return os.str();
}
