#ifndef BMSBEL_PARSER_H_
#define BMSBEL_PARSER_H_

#include <vector>
#include <stack>
#include <queue>

#include "bms_define.h"
#include "bms_bms.h"
#include "bms_random_manager.h"
#include "bms_text_file_reader.h"
#include "bms_handler.h"

class BmsParser;

// -- BmsParseFrame ------------------------------------------------------
struct BmsParseFrame {
public:

  explicit BmsParseFrame( BmsBms* master, BmsRandomManager& random_manager );

  explicit BmsParseFrame( BmsParseFrame*               parent,
                          BmsBms*                      current,
                          BmsRandomIfStatementElement* current_if_statement,
                          bool                         master_take_over );

  ~BmsParseFrame();

  template <class TYPE>
  TYPE* GetChildElementAs( void ) {
    return dynamic_cast<TYPE*>( current_if_statement_->GetChildren()[if_statement_position_] );
  }

public:
  BmsParseFrame*               parent_;
  BmsBms*                      master_;
  std::vector<unsigned int>*   master_bgm_channel_count_;
  BmsBms*                      current_;
  std::vector<unsigned int>*   current_bgm_channel_count_;
  BmsRandomIfStatementElement* current_if_statement_;
  unsigned int                 if_statement_position_;
};


// -- BmsParser ----------------------------------------------------------
class BmsParser {
public:
  explicit BmsParser( void );

  void Parse( const std::string& filename, BmsBms& bms, BmsRandomManager& random_manager );
  void Parse( BmsTextFileReader& reader, BmsBms& bms, BmsRandomManager& random_manager );

  void SetDuplicateHeaderHandler( BmsHandler::DuplicateHeader* handler );
  void SetDuplicateBarChangeHandler( BmsHandler::DuplicateBarChange* handler );
  void SetParseErrorIgnoreHandler( BmsHandler::ParseErrorIgnore* handler );
  void SetGenerateRandomHandler( BmsHandler::GenerateRandom* handler );

  unsigned int GetLineNumber( void ) const;

private:
  void ParseDefault( const char* str );
  void ParseComment( const char* str );
  void ParseHeaderOrBar( const char* str );
  void ParseHeaderKey( const char* str );
  void ParseHeaderSeparator( const char* str );
  void ParseHeaderValue( const char* str );
  void ParseRegistArrayValue( BmsRegistArray& (BmsBms::*func)( void ) );
  void ParseBar( const char* str );
  void ParseChannel( const char* str );
  void ParseColon( const char* str );
  void ParseBarChangeValue( const char* str );
  void ParseObjectArray( const char* str );
  void ParseObjectArrayValue( BmsBms& bms,
                               std::vector<unsigned int>& bgm_channel_count,
                               const std::vector<BmsWord>& word_array );
  void ParseRandomStatement( void );

  unsigned int DecideBarDivisionCount( unsigned int current, unsigned int request );

private:
  BmsHandler::DuplicateHeader*    duplicate_header_handler_;
  BmsHandler::DuplicateBarChange* duplicate_bar_change_handler_;
  BmsHandler::ParseErrorIgnore*   parse_error_ignore_handler_;
  BmsHandler::GenerateRandom*     generate_random_handler_;

  bool                       is_first_parse_;
  std::vector<std::string>   data_;
  unsigned int               line_number_;
  BmsRandomManager*          random_manager_;
  std::queue<unsigned int>   random_value_queue_;
  std::stack<BmsParseFrame*> frame_stack_;

  unsigned int              bar_;
  BmsWord                   channel_;
  std::string               key_;
  std::string               value_;
};

#endif // BMSBEL_PARSER_H_
